@@ -1,9 +1,8 @@
 ;;; greek.el --- Quail package for inputting Greek -*-coding: iso-2022-7bit-*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 2001-2013 Free Software Foundation, Inc.
 ;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008
+;;   2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
 ;;   Registration Number H14PRO021
 
@@ -11,10 +10,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -155,7 +152,7 @@ eta		H		h
 theta		Q		q
 iota		I		i
 kappa		K		k
-lamda		L		l
+lambda		L		l
 mu		M		m
 nu		N		n
 xi		X		x
@@ -436,7 +433,7 @@ eta		H		h
 theta		J		j
 iota		I		i
 kappa		K		k
-lamda		L		l
+lambda		L		l
 mu		M		m
 nu		N		n
 xi		X		x
@@ -1150,7 +1147,7 @@ nil t t nil nil nil nil nil nil nil t)
 
 In the right of ,Fk(B key is a combination key, where
  ,F4(B acute
- ,F((B diaresis
+ ,F((B diaeresis
 
 e.g.
  ,F4(B + ,Fa(B -> ,F\(B
@@ -1281,7 +1278,9 @@ e.g.
  (";:i" ?,F@(B)
  (":;i" ?,F@(B)
  (";:y" ?,F`(B)
- (":;y" ?,F`(B))
+ (":;y" ?,F`(B)
+ (";<" ?$(Q)((B)
+ (";>" ?$(Q)2(B))
 
 (quail-define-package
  "greek-postfix" "GreekPost" ",FX(B" nil
@@ -1290,7 +1289,7 @@ e.g.
 
 In the right of ,Fk(B key is a combination key, where
  ,F4(B acute
- ,F((B diaresis
+ ,F((B diaeresis
 
 e.g.
  ,Fa(B + ,F4(B -> ,F\(B
@@ -1421,8 +1420,12 @@ e.g.
  ("i:;" ?,F@(B)
  ("i;:" ?,F@(B)
  ("y:;" ?,F`(B)
- ("y;:" ?,F`(B))
+ ("y;:" ?,F`(B)
+ ;; These two are asymmetric with ";<" and ";>" in "greek" input
+ ;; method.  But, as the other Latin postfix methods adopt "<<" and
+ ;; ">>", it may be better to follow them.
+ ("<<" ?$(Q)((B)
+ (">>" ?$(Q)2(B))
 
 
-;;; arch-tag: 2a37e042-db1b-4ecf-b755-117775a3c150
 ;;; greek.el ends here
