@@ -1,6 +1,7 @@
 /* pop.c: client routines for talking to a POP3-protocol post-office server
-   Copyright (C) 1991, 1993, 1996, 1997, 1999, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+
+Copyright (C) 1991, 1993, 1996-1997, 1999, 2001-2013 Free Software
+Foundation, Inc.
 
 Author: Jonathan Kamens <jik@security.ov.com>
 
@@ -20,11 +21,7 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#else
-#define MAIL_USE_POP
-#endif
 
 #ifdef MAIL_USE_POP
 
@@ -33,15 +30,15 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #include "ntlib.h"
 #include <winsock.h>
 #undef SOCKET_ERROR
-#define RECV(s,buf,len,flags) recv(s,buf,len,flags)
-#define SEND(s,buf,len,flags) send(s,buf,len,flags)
-#define CLOSESOCKET(s) closesocket(s)
+#define RECV(s,buf,len,flags) recv (s,buf,len,flags)
+#define SEND(s,buf,len,flags) send (s,buf,len,flags)
+#define CLOSESOCKET(s) closesocket (s)
 #else
 #include <netinet/in.h>
 #include <sys/socket.h>
-#define RECV(s,buf,len,flags) read(s,buf,len)
-#define SEND(s,buf,len,flags) write(s,buf,len)
-#define CLOSESOCKET(s) close(s)
+#define RECV(s,buf,len,flags) read (s,buf,len)
+#define SEND(s,buf,len,flags) write (s,buf,len)
+#define CLOSESOCKET(s) close (s)
 #endif
 #include <pop.h>
 
@@ -64,13 +61,8 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 #include <netdb.h>
 #include <errno.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
 #include <string.h>
-#define index strchr
-#endif
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 #ifdef KERBEROS
 # ifdef HAVE_KRB5_H
@@ -92,6 +84,8 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 # endif
 #endif /* KERBEROS */
 
+#include <min-max.h>
+
 #ifdef KERBEROS
 #ifndef KERBEROS5
 extern int krb_sendauth (/* long, int, KTEXT, char *, char *, char *,
@@ -103,47 +97,35 @@ extern char *krb_realmofhost (/* char * */);
 #endif /* KERBEROS */
 
 #ifndef WINDOWSNT
-#if !defined(HAVE_H_ERRNO) || !defined(HAVE_CONFIG_H)
+#ifndef HAVE_H_ERRNO
 extern int h_errno;
 #endif
 #endif
 
-#ifndef __P
-# ifdef __STDC__
-#  define __P(a) a
-# else
-#  define __P(a) ()
-# endif /* __STDC__ */
-#endif /* ! __P */
-
-static int socket_connection __P((char *, int));
-static int pop_getline __P((popserver, char **));
-static int sendline __P((popserver, char *));
-static int fullwrite __P((int, char *, int));
-static int getok __P((popserver));
+static int socket_connection (char *, int);
+static int pop_getline (popserver, char **);
+static int sendline (popserver, const char *);
+static int fullwrite (int, char *, int);
+static int getok (popserver);
 #if 0
-static int gettermination __P((popserver));
+static int gettermination (popserver);
 #endif
-static void pop_trash __P((popserver));
-static char *find_crlf __P((char *, int));
+static void pop_trash (popserver);
+static char *find_crlf (char *, int);
 
 #define ERROR_MAX 160		/* a pretty arbitrary size, but needs
 				   to be bigger than the original
 				   value of 80 */
 #define POP_PORT 110
-#define KPOP_PORT 1109
 #define POP_SERVICE "pop3"	/* we don't want the POP2 port! */
 #ifdef KERBEROS
+#define KPOP_PORT 1109
 #define KPOP_SERVICE "kpop"	/* never used: look for 20060515 to see why */
 #endif
 
 char pop_error[ERROR_MAX];
 int pop_debug = 0;
 
-#ifndef min
-#define min(a,b) (((a) < (b)) ? (a) : (b))
-#endif
-
 /*
  * Function: pop_open (char *host, char *username, char *password,
  * 		       int flags)
@@ -174,11 +156,7 @@ int pop_debug = 0;
  * 	explanation of the error.
  */
 popserver
-pop_open (host, username, password, flags)
-     char *host;
-     char *username;
-     char *password;
-     int flags;
+pop_open (char *host, char *username, char *password, int flags)
 {
   int sock;
   popserver server;
@@ -345,10 +323,7 @@ pop_open (host, username, password, flags)
  * 	connection impossible.
  */
 int
-pop_stat (server, count, size)
-     popserver server;
-     int *count;
-     int *size;
+pop_stat (popserver server, int *count, int *size)
 {
   char *fromserver;
   char *end_ptr;
@@ -365,9 +340,7 @@ pop_stat (server, count, size)
   if (strncmp (fromserver, "+OK ", 4))
     {
       if (0 == strncmp (fromserver, "-ERR", 4))
-	{
-	  strncpy (pop_error, fromserver, ERROR_MAX);
-	}
+	snprintf (pop_error, ERROR_MAX, "%s", fromserver);
       else
 	{
 	  strcpy (pop_error,
@@ -421,11 +394,7 @@ pop_stat (server, count, size)
  * 	connection impossible.
  */
 int
-pop_list (server, message, IDs, sizes)
-     popserver server;
-     int message;
-     int **IDs;
-     int **sizes;
+pop_list (popserver server, int message, int **IDs, int **sizes)
 {
   int how_many, i;
   char *fromserver;
@@ -472,7 +441,7 @@ pop_list (server, message, IDs, sizes)
       if (strncmp (fromserver, "+OK ", 4))
 	{
 	  if (! strncmp (fromserver, "-ERR", 4))
-	    strncpy (pop_error, fromserver, ERROR_MAX);
+	    snprintf (pop_error, ERROR_MAX, "%s", fromserver);
 	  else
 	    {
 	      strcpy (pop_error,
@@ -484,7 +453,7 @@ pop_list (server, message, IDs, sizes)
 	  return (-1);
 	}
       (*IDs)[0] = atoi (&fromserver[4]);
-      fromserver = index (&fromserver[4], ' ');
+      fromserver = strchr (&fromserver[4], ' ');
       if (! fromserver)
 	{
 	  strcpy (pop_error,
@@ -515,7 +484,7 @@ pop_list (server, message, IDs, sizes)
 	      return (-1);
 	    }
 	  (*IDs)[i] = atoi (fromserver);
-	  fromserver = index (fromserver, ' ');
+	  fromserver = strchr (fromserver, ' ');
 	  if (! fromserver)
 	    {
 	      strcpy (pop_error,
@@ -567,11 +536,7 @@ pop_list (server, message, IDs, sizes)
  * Side effects: May kill connection on error.
  */
 int
-pop_retrieve (server, message, markfrom, msg_buf)
-     popserver server;
-     int message;
-     int markfrom;
-     char **msg_buf;
+pop_retrieve (popserver server, int message, int markfrom, char **msg_buf)
 {
   int *IDs, *sizes, bufsize, fromcount = 0, cp = 0;
   char *ptr, *fromserver;
@@ -635,7 +600,7 @@ pop_retrieve (server, message, markfrom, msg_buf)
 	    }
 	  ptr[cp++] = '>';
 	}
-      bcopy (fromserver, &ptr[cp], ret);
+      memcpy (&ptr[cp], fromserver, ret);
       cp += ret;
       ptr[cp++] = '\n';
     }
@@ -645,10 +610,7 @@ pop_retrieve (server, message, markfrom, msg_buf)
 }
 
 int
-pop_retrieve_first (server, message, response)
-     popserver server;
-     int message;
-     char **response;
+pop_retrieve_first (popserver server, int message, char **response)
 {
   sprintf (pop_error, "RETR %d", message);
   return (pop_multi_first (server, pop_error, response));
@@ -663,25 +625,19 @@ pop_retrieve_first (server, message, response)
   */
 
 int
-pop_retrieve_next (server, line)
-     popserver server;
-     char **line;
+pop_retrieve_next (popserver server, char **line)
 {
   return (pop_multi_next (server, line));
 }
 
 int
-pop_retrieve_flush (server)
-     popserver server;
+pop_retrieve_flush (popserver server)
 {
   return (pop_multi_flush (server));
 }
 
 int
-pop_top_first (server, message, lines, response)
-     popserver server;
-     int message, lines;
-     char **response;
+pop_top_first (popserver server, int message, int lines, char **response)
 {
   sprintf (pop_error, "TOP %d %d", message, lines);
   return (pop_multi_first (server, pop_error, response));
@@ -696,25 +652,19 @@ pop_top_first (server, message, lines, response)
   */
 
 int
-pop_top_next (server, line)
-     popserver server;
-     char **line;
+pop_top_next (popserver server, char **line)
 {
   return (pop_multi_next (server, line));
 }
 
 int
-pop_top_flush (server)
-     popserver server;
+pop_top_flush (popserver server)
 {
   return (pop_multi_flush (server));
 }
 
 int
-pop_multi_first (server, command, response)
-     popserver server;
-     char *command;
-     char **response;
+pop_multi_first (popserver server, const char *command, char **response)
 {
   if (server->in_multi)
     {
@@ -730,7 +680,7 @@ pop_multi_first (server, command, response)
 
   if (0 == strncmp (*response, "-ERR", 4))
     {
-      strncpy (pop_error, *response, ERROR_MAX);
+      snprintf (pop_error, ERROR_MAX, "%s", *response);
       return (-1);
     }
   else if (0 == strncmp (*response, "+OK", 3))
@@ -757,9 +707,7 @@ pop_multi_first (server, command, response)
   0, LINE is set to null. */
 
 int
-pop_multi_next (server, line)
-     popserver server;
-     char **line;
+pop_multi_next (popserver server, char **line)
 {
   char *fromserver;
   int ret;
@@ -797,8 +745,7 @@ pop_multi_next (server, line)
 }
 
 int
-pop_multi_flush (server)
-     popserver server;
+pop_multi_flush (popserver server)
 {
   char *line;
   int ret;
@@ -829,9 +776,7 @@ pop_multi_flush (server)
  * 	otherwise.
  */
 int
-pop_delete (server, message)
-     popserver server;
-     int message;
+pop_delete (popserver server, int message)
 {
   if (server->in_multi)
     {
@@ -861,8 +806,7 @@ pop_delete (server, message)
  * Side effects: Closes connection on error.
  */
 int
-pop_noop (server)
-     popserver server;
+pop_noop (popserver server)
 {
   if (server->in_multi)
     {
@@ -891,8 +835,7 @@ pop_noop (server)
  * Side effects: Closes the connection on error.
  */
 int
-pop_last (server)
-     popserver server;
+pop_last (popserver server)
 {
   char *fromserver;
 
@@ -910,7 +853,7 @@ pop_last (server)
 
   if (! strncmp (fromserver, "-ERR", 4))
     {
-      strncpy (pop_error, fromserver, ERROR_MAX);
+      snprintf (pop_error, ERROR_MAX, "%s", fromserver);
       return (-1);
     }
   else if (strncmp (fromserver, "+OK ", 4))
@@ -949,8 +892,7 @@ pop_last (server)
  * Side effects: Closes the connection on error.
  */
 int
-pop_reset (server)
-     popserver server;
+pop_reset (popserver server)
 {
   if (pop_retrieve_flush (server))
     {
@@ -978,8 +920,7 @@ pop_reset (server)
  * 	function is called, even if an error occurs.
  */
 int
-pop_quit (server)
-     popserver server;
+pop_quit (popserver server)
 {
   int ret = 0;
 
@@ -1023,9 +964,7 @@ static int have_winsock = 0;
  * 	into pop_error.
  */
 static int
-socket_connection (host, flags)
-     char *host;
-     int flags;
+socket_connection (char *host, int flags)
 {
 #ifdef HAVE_GETADDRINFO
   struct addrinfo *res, *it;
@@ -1037,7 +976,7 @@ socket_connection (host, flags)
   struct servent *servent;
   struct sockaddr_in addr;
   char found_port = 0;
-  char *service;
+  const char *service;
   int sock;
   char *realhost;
 #ifdef KERBEROS
@@ -1069,7 +1008,7 @@ socket_connection (host, flags)
   }
 #endif
 
-  bzero ((char *) &addr, sizeof (addr));
+  memset (&addr, 0, sizeof (addr));
   addr.sin_family = AF_INET;
 
   /** "kpop" service is  never used: look for 20060515 to see why **/
@@ -1114,15 +1053,14 @@ socket_connection (host, flags)
   sock = socket (PF_INET, SOCK_STREAM, 0);
   if (sock < 0)
     {
-      strcpy (pop_error, POP_SOCKET_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (POP_SOCKET_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s",
+		POP_SOCKET_ERROR, strerror (errno));
       return (-1);
 
     }
 
 #ifdef HAVE_GETADDRINFO
-  memset (&hints, 0, sizeof(hints));
+  memset (&hints, 0, sizeof (hints));
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_flags = AI_CANONNAME;
   hints.ai_family = AF_INET;
@@ -1145,8 +1083,7 @@ socket_connection (host, flags)
           if (it->ai_addrlen == sizeof (addr))
             {
               struct sockaddr_in *in_a = (struct sockaddr_in *) it->ai_addr;
-              bcopy (&in_a->sin_addr, (char *) &addr.sin_addr,
-                     sizeof (addr.sin_addr));
+              addr.sin_addr = in_a->sin_addr;
               if (! connect (sock, (struct sockaddr *) &addr, sizeof (addr)))
                 break;
             }
@@ -1174,8 +1111,7 @@ socket_connection (host, flags)
 
   while (*hostent->h_addr_list)
     {
-      bcopy (*hostent->h_addr_list, (char *) &addr.sin_addr,
-	     hostent->h_length);
+      memcpy (&addr.sin_addr, *hostent->h_addr_list, hostent->h_length);
       if (! connect (sock, (struct sockaddr *) &addr, sizeof (addr)))
 	break;
       hostent->h_addr_list++;
@@ -1194,9 +1130,7 @@ socket_connection (host, flags)
   if (! connect_ok)
     {
       CLOSESOCKET (sock);
-      strcpy (pop_error, CONNECT_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (CONNECT_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s", CONNECT_ERROR, strerror (errno));
       return (-1);
 
     }
@@ -1214,9 +1148,8 @@ socket_connection (host, flags)
 	    krb5_auth_con_free (kcontext, auth_context);
 	  if (kcontext)
 	    krb5_free_context (kcontext);
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, error_message (rem),
-		   ERROR_MAX - sizeof(KRB_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s",
+		    KRB_ERROR, error_message (rem));
 	  CLOSESOCKET (sock);
 	  return (-1);
 	}
@@ -1254,30 +1187,19 @@ socket_connection (host, flags)
       krb5_free_principal (kcontext, server);
       if (rem)
 	{
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, error_message (rem),
-		   ERROR_MAX - sizeof (KRB_ERROR));
+	  int pop_error_len = snprintf (pop_error, ERROR_MAX, "%s%s",
+					KRB_ERROR, error_message (rem));
 #if defined HAVE_KRB5_ERROR_TEXT
 	  if (err_ret && err_ret->text.length)
 	    {
-	      strncat (pop_error, " [server says '",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, err_ret->text.data,
-		       min (ERROR_MAX - strlen (pop_error) - 1,
-			    err_ret->text.length));
-	      strncat (pop_error, "']",
-		       ERROR_MAX - strlen (pop_error) - 1);
+	      int errlen = err_ret->text.length;
+	      snprintf (pop_error + pop_error_len, ERROR_MAX - pop_error_len,
+			" [server says '.*%s']", errlen, err_ret->text.data);
 	    }
 #elif defined HAVE_KRB5_ERROR_E_TEXT
-	  if (err_ret && err_ret->e_text && strlen(*err_ret->e_text))
-	    {
-	      strncat (pop_error, " [server says '",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, *err_ret->e_text,
-		       ERROR_MAX - strlen (pop_error) - 1);
-	      strncat (pop_error, "']",
-		       ERROR_MAX - strlen (pop_error) - 1);
-	    }
+	  if (err_ret && err_ret->e_text && **err_ret->e_text)
+	    snprintf (pop_error + pop_error_len, ERRMAX - pop_error_len,
+		      " [server says '%s']", *err_ret->e_text);
 #endif
 	  if (err_ret)
 	    krb5_free_error (kcontext, err_ret);
@@ -1298,9 +1220,7 @@ socket_connection (host, flags)
       free ((char *) ticket);
       if (rem != KSUCCESS)
 	{
-	  strcpy (pop_error, KRB_ERROR);
-	  strncat (pop_error, krb_err_txt[rem],
-		   ERROR_MAX - sizeof (KRB_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s", KRB_ERROR, krb_err_txt[rem]);
 	  CLOSESOCKET (sock);
 	  return (-1);
 	}
@@ -1335,9 +1255,7 @@ socket_connection (host, flags)
  * THE RETURNED LINE MAY CONTAIN EMBEDDED NULLS!
  */
 static int
-pop_getline (server, line)
-     popserver server;
-     char **line;
+pop_getline (popserver server, char **line)
 {
 #define GETLINE_ERROR "Error reading from server: "
 
@@ -1369,8 +1287,8 @@ pop_getline (server, line)
 	}
       else
 	{
-	  bcopy (server->buffer + server->buffer_index,
-		 server->buffer, server->data);
+	  memmove (server->buffer, server->buffer + server->buffer_index,
+		   server->data);
 	  /* Record the fact that we've searched the data already in
              the buffer for a CRLF, so that when we search below, we
              don't have to search the same data twice.  There's a "-
@@ -1407,9 +1325,8 @@ pop_getline (server, line)
 		  server->buffer_size - server->data - 1, 0);
       if (ret < 0)
 	{
-	  strcpy (pop_error, GETLINE_ERROR);
-	  strncat (pop_error, strerror (errno),
-		   ERROR_MAX - sizeof (GETLINE_ERROR));
+	  snprintf (pop_error, ERROR_MAX, "%s%s",
+		    GETLINE_ERROR, strerror (errno));
 	  pop_trash (server);
 	  return (-1);
 	}
@@ -1467,9 +1384,7 @@ pop_getline (server, line)
  * Side effects: Closes the connection on error.
  */
 static int
-sendline (server, line)
-     popserver server;
-     char *line;
+sendline (popserver server, const char *line)
 {
 #define SENDLINE_ERROR "Error writing to POP server: "
   int ret;
@@ -1495,9 +1410,7 @@ sendline (server, line)
   if (ret < 0)
     {
       pop_trash (server);
-      strcpy (pop_error, SENDLINE_ERROR);
-      strncat (pop_error, strerror (errno),
-	       ERROR_MAX - sizeof (SENDLINE_ERROR));
+      snprintf (pop_error, ERROR_MAX, "%s%s", SENDLINE_ERROR, strerror (errno));
       return (ret);
     }
 
@@ -1516,10 +1429,7 @@ sendline (server, line)
  * Return value: Same as write.  Pop_error is not set.
  */
 static int
-fullwrite (fd, buf, nbytes)
-     int fd;
-     char *buf;
-     int nbytes;
+fullwrite (int fd, char *buf, int nbytes)
 {
   char *cp;
   int ret = 0;
@@ -1549,8 +1459,7 @@ fullwrite (fd, buf, nbytes)
  * Side effects: On failure, may make the connection unusable.
  */
 static int
-getok (server)
-     popserver server;
+getok (popserver server)
 {
   char *fromline;
 
@@ -1563,8 +1472,7 @@ getok (server)
     return (0);
   else if (! strncmp (fromline, "-ERR", 4))
     {
-      strncpy (pop_error, fromline, ERROR_MAX);
-      pop_error[ERROR_MAX-1] = '\0';
+      snprintf (pop_error, ERROR_MAX, "%s", fromline);
       return (-1);
     }
   else
@@ -1621,8 +1529,7 @@ gettermination (server)
  * 	since the last pop_reset) may be lost.
  */
 void
-pop_close (server)
-     popserver server;
+pop_close (popserver server)
 {
   pop_trash (server);
   free ((char *) server);
@@ -1638,8 +1545,7 @@ pop_close (server)
  * 	pop_close or pop_quit after this function has been called.
  */
 static void
-pop_trash (server)
-     popserver server;
+pop_trash (popserver server)
 {
   if (server->file >= 0)
     {
@@ -1671,9 +1577,7 @@ pop_trash (server)
    null, or 0 if it does not contain one.  */
 
 static char *
-find_crlf (in_string, len)
-     char *in_string;
-     int len;
+find_crlf (char *in_string, int len)
 {
   while (len--)
     {
@@ -1689,6 +1593,3 @@ find_crlf (in_string, len)
 }
 
 #endif /* MAIL_USE_POP */
-
-/* arch-tag: ceb37041-b7ad-49a8-a63d-286618b8367d
-   (do not change this comment) */
