@@ -1,7 +1,6 @@
 ;;; calccomp.el --- composition functions for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2013 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
@@ -50,19 +49,19 @@
 ;;;
 ;;;    (tag X C)             Composition C corresponds to sub-expression X
 
-;; math-comp-just and math-comp-comma-spc are local to 
-;; math-compose-expr, but are used by math-compose-matrix, which is 
+;; math-comp-just and math-comp-comma-spc are local to
+;; math-compose-expr, but are used by math-compose-matrix, which is
 ;; called by math-compose-expr
 (defvar math-comp-just)
 (defvar math-comp-comma-spc)
 
-;; math-comp-vector-prec is local to math-compose-expr, but is used by 
-;; math-compose-matrix and math-compose-rows, which are called by 
+;; math-comp-vector-prec is local to math-compose-expr, but is used by
+;; math-compose-matrix and math-compose-rows, which are called by
 ;; math-compose-expr.
 (defvar math-comp-vector-prec)
 
-;; math-comp-left-bracket, math-comp-right-bracket and math-comp-comma are 
-;; local to math-compose-expr, but are used by math-compose-rows, which is 
+;; math-comp-left-bracket, math-comp-right-bracket and math-comp-comma are
+;; local to math-compose-expr, but are used by math-compose-rows, which is
 ;; called by math-compose-expr.
 (defvar math-comp-left-bracket)
 (defvar math-comp-right-bracket)
@@ -87,8 +86,11 @@
           (setq sn (math-to-underscores sn)))
       sn)))
 
-(defun math-compose-expr (a prec)
-  (let ((math-compose-level (1+ math-compose-level))
+;;; Give multiplication precedence when composing to avoid
+;;; writing a*(b c) instead of a b c
+(defun math-compose-expr (a prec &optional div)
+  (let ((calc-multiplication-has-precedence t)
+        (math-compose-level (1+ math-compose-level))
         (math-expr-opers (math-expr-ops))
         spfn)
     (cond
@@ -100,7 +102,7 @@
 	(list 'tag a (math-compose-expr a prec))))
      ((and (not (consp a)) (not (integerp a)))
       (concat "'" (prin1-to-string a)))
-     ((setq spfn (assq (car-safe a) 
+     ((setq spfn (assq (car-safe a)
                        (get calc-language 'math-special-function-table)))
       (setq spfn (cdr spfn))
       (if (consp spfn)
@@ -111,12 +113,12 @@
 	      (and (nth 1 calc-frac-format) (Math-integerp a)))
 	  (if (and
                calc-language
-               (not (memq calc-language 
+               (not (memq calc-language
                           '(flat big unform))))
 	      (let ((aa (math-adjust-fraction a))
 		    (calc-frac-format nil))
 		(math-compose-expr (list '/
-					 (if (memq calc-language 
+					 (if (memq calc-language
                                                    calc-lang-slash-idiv)
 					     (math-float (nth 1 aa))
 					   (nth 1 aa))
@@ -281,22 +283,22 @@
 					    (cdr a)
 					    (if full rows 3) t)))))
 	      (if (or calc-full-vectors (< (length a) 7))
-                  (if (and 
+                  (if (and
                        (setq spfn (get calc-language 'math-matrix-formatter))
                        (math-matrixp a))
                       (funcall spfn a)
                     (list 'horiz
                           math-comp-left-bracket
-                          (math-compose-vector (cdr a) 
+                          (math-compose-vector (cdr a)
                                                (concat math-comp-comma " ")
                                                math-comp-vector-prec)
                           math-comp-right-bracket))
 		(list 'horiz
 		      math-comp-left-bracket
 		      (math-compose-vector (list (nth 1 a) (nth 2 a) (nth 3 a))
-					   (concat math-comp-comma " ") 
+					   (concat math-comp-comma " ")
                                            math-comp-vector-prec)
-		      math-comp-comma 
+		      math-comp-comma
                       (if (setq spfn (get calc-language 'math-dots))
                           (concat " " spfn)
                         " ...")
@@ -592,7 +594,9 @@
 		    (or (= (length a) 3) (eq (car a) 'calcFunc-if))
 		    (/= (nth 3 op) -1))
 	       (cond
-		((> prec (or (nth 4 op) (min (nth 2 op) (nth 3 op))))
+		((or
+                  (> prec (or (nth 4 op) (min (nth 2 op) (nth 3 op))))
+                  (and div (eq (car a) '*)))
 		 (if (and (memq calc-language '(tex latex))
 			  (not (math-tex-expr-is-flat a)))
 		     (if (eq (car-safe a) '/)
@@ -632,7 +636,7 @@
 				      nil)
 				  math-compose-level))
 			(lhs (math-compose-expr (nth 1 a) (nth 2 op)))
-			(rhs (math-compose-expr (nth 2 a) (nth 3 op))))
+			(rhs (math-compose-expr (nth 2 a) (nth 3 op) (eq (nth 1 op) '/))))
 		   (and (equal (car op) "^")
 			(eq (math-comp-first-char lhs) ?-)
 			(setq lhs (list 'horiz "(" lhs ")")))
@@ -663,6 +667,8 @@
 			      (and prevc nextc
 				   (or (and (>= nextc ?a) (<= nextc ?z))
 				       (and (>= nextc ?A) (<= nextc ?Z))
+				       (and (>= nextc ?α) (<= nextc ?ω))
+				       (and (>= nextc ?Α) (<= nextc ?Ω))
 				       (and (>= nextc ?0) (<= nextc ?9))
 				       (memq nextc '(?. ?_ ?#
 							?\( ?\[ ?\{))
@@ -732,7 +738,7 @@
 				(not (math-tex-expr-is-flat (nth 1 a))))))
 		   (list 'horiz
 			 (if lr "\\left" "")
-			 (if (string-match "\\`u\\([^a-zA-Z]\\)\\'" (car op))
+			 (if (string-match "\\`u\\([^a-zA-Zα-ωΑ-Ω]\\)\\'" (car op))
 			     (substring (car op) 1)
 			   (car op))
 			 (if (or lr (> (length (car op)) 2)) " " "")
@@ -758,7 +764,7 @@
 		(t
 		 (let ((rhs (math-compose-expr (nth 1 a) (nth 3 op))))
 		   (list 'horiz
-			 (let ((ops (if (string-match "\\`u\\([^a-zA-Z]\\)\\'"
+			 (let ((ops (if (string-match "\\`u\\([^a-zA-Zα-ωΑ-Ω]\\)\\'"
 						      (car op))
 					(substring (car op) 1)
 				      (car op))))
@@ -806,7 +812,7 @@
 		     (setq func (car func2)))
 		 (setq func (math-remove-dashes
 			     (if (string-match
-				  "\\`calcFunc-\\([a-zA-Z0-9']+\\)\\'"
+				  "\\`calcFunc-\\([a-zA-Zα-ωΑ-Ω0-9']+\\)\\'"
 				  (symbol-name func))
 				 (math-match-substring (symbol-name func) 1)
 			       (symbol-name func))))
@@ -867,7 +873,7 @@
 						    math-comp-vector-prec)
 						   (if (= col cols)
 						       ""
-						     (concat 
+						     (concat
                                                       math-comp-comma-spc " ")))))
 					  a)))
 		      res)))
@@ -878,7 +884,7 @@
       (if (<= count 0)
 	  (if (< count 0)
 	      (math-compose-rows (cdr a) -1 nil)
-	    (cons (concat 
+	    (cons (concat
                    (let ((mdots (get calc-language 'math-dots)))
                      (if mdots
                          (concat " " mdots)
@@ -1117,7 +1123,7 @@
 	       (if (memq prec '(196 201)) ")" "")))))
 
 ;; The variables math-svo-c, math-svo-wid and math-svo-off are local
-;; to math-stack-value-offset in calc.el, but are used by 
+;; to math-stack-value-offset in calc.el, but are used by
 ;; math-stack-value-offset-fancy, which is called by math-stack-value-offset..
 (defvar math-svo-c)
 (defvar math-svo-wid)
@@ -1193,11 +1199,11 @@
 ;;; of the formula.
 
 ;; The variables math-comp-full-width, math-comp-highlight, math-comp-word,
-;; math-comp-level, math-comp-margin and math-comp-buf are local to 
-;; math-comp-to-string-flat, but are used by math-comp-to-string-flat-term, 
+;; math-comp-level, math-comp-margin and math-comp-buf are local to
+;; math-comp-to-string-flat, but are used by math-comp-to-string-flat-term,
 ;; which is called by math-comp-to-string-flat.
-;; math-comp-highlight and math-comp-buf are also local to 
-;; math-comp-simplify-term and math-comp-simplify respectively, but are used 
+;; math-comp-highlight and math-comp-buf are also local to
+;; math-comp-simplify-term and math-comp-simplify respectively, but are used
 ;; by math-comp-add-string.
 (defvar math-comp-full-width)
 (defvar math-comp-highlight)
@@ -1242,7 +1248,7 @@
   (cond ((not (consp c))
 	 (if math-comp-highlight
 	     (setq c (math-comp-highlight-string c)))
-	 (setq math-comp-word (if (= (length math-comp-word) 0) c 
+	 (setq math-comp-word (if (= (length math-comp-word) 0) c
                                 (concat math-comp-word c))
 	       math-comp-pos (+ math-comp-pos (length c))))
 
@@ -1281,12 +1287,7 @@
 		 (let ((prefix "") mrg wid)
 		   (setq mrg (aref math-comp-buf-margin i))
 		   (if (> mrg 12)  ; indenting too far, go back to far left
-		       (let ((j i) (new (if calc-line-numbering 5 1)))
-			 '(while (<= j math-comp-level)
-			   (aset math-comp-buf-margin j
-				 (+ (aref math-comp-buf-margin j) (- new mrg)))
-			   (setq j (1+ j)))
-			 (setq mrg new)))
+		       (setq mrg (if calc-line-numbering 5 1)))
 		   (setq wid (+ (length str) math-comp-margin))
 		   (and (> (length str) 0) (= (aref str 0) ? )
 			(> (length math-comp-buf) 0)
@@ -1337,16 +1338,19 @@
 
 (defun math-comp-highlight-string (s)
   (setq s (copy-sequence s))
-  (let ((i (length s)))
-    (while (>= (setq i (1- i)) 0)
-      (or (memq (aref s i) '(32 ?\n))
-	  (aset s i (if calc-show-selections ?\. ?\#)))))
-  s)
-
+  (if calc-highlight-selections-with-faces
+      (if (not calc-show-selections)
+          (propertize s 'face 'calc-selected-face)
+        (propertize s 'face 'calc-nonselected-face))
+    (let ((i (length s)))
+      (while (>= (setq i (1- i)) 0)
+        (or (memq (aref s i) '(32 ?\n))
+            (aset s i (if calc-show-selections ?\. ?\#)))))
+    s))
 
 ;; The variable math-comp-sel-tag is local to calc-find-selected-part
-;; in calc-sel.el, but is used by math-comp-sel-flat-term and 
-;; math-comp-add-string-sel, which are called (indirectly) by 
+;; in calc-sel.el, but is used by math-comp-sel-flat-term and
+;; math-comp-add-string-sel, which are called (indirectly) by
 ;; calc-find-selected-part.
 (defvar math-comp-sel-tag)
 
@@ -1666,5 +1670,8 @@
 
 (provide 'calccomp)
 
-;; arch-tag: 7c45d10a-a286-4dab-af49-7ae8989fbf78
+;; Local variables:
+;; coding: utf-8
+;; End:
+
 ;;; calccomp.el ends here
