@@ -1,10 +1,11 @@
 ;;; cedet-idutils.el --- ID Utils support for CEDET.
 
-;; Copyright (C) 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2013 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <eric@siege-engine.com>
 ;; Version: 0.2
 ;; Keywords: OO, lisp
+;; Package: cedet
 
 ;; This file is part of GNU Emacs.
 
@@ -83,7 +84,7 @@ Note: Scope is not yet supported."
 
 (defun cedet-idutils-fnid-call (flags)
   "Call ID Utils fnid with the list of FLAGS.
-Return the created buffer with with program output."
+Return the created buffer with program output."
   (let ((b (get-buffer-create "*CEDET fnid*"))
 	(cd default-directory)
 	)
@@ -111,7 +112,7 @@ Return the created buffer with with program output."
 
 (defun cedet-idutils-mkid-call (flags)
   "Call ID Utils mkid with the list of FLAGS.
-Return the created buffer with with program output."
+Return the created buffer with program output."
   (let ((b (get-buffer-create "*CEDET mkid*"))
 	(cd default-directory)
 	)
@@ -162,8 +163,8 @@ the error code."
 
 (defun cedet-idutils-version-check (&optional noerror)
   "Check the version of the installed ID Utils command.
-If optional programatic argument NOERROR is non-nil, then
-instead of throwing an error if Global isn't available, then
+If optional programmatic argument NOERROR is non-nil,
+then instead of throwing an error if Global isn't available,
 return nil."
   (interactive)
   (require 'inversion)
@@ -178,8 +179,9 @@ return nil."
 	  nil)
       (with-current-buffer b
 	(goto-char (point-min))
-	(re-search-forward "fnid - \\([0-9.]+\\)" nil t)
-	(setq rev (match-string 1))
+	(if (re-search-forward "fnid - \\([0-9.]+\\)" nil t)
+	    (setq rev (match-string 1))
+	  (setq rev "0"))
 	(if (inversion-check-version rev nil cedet-idutils-min-version)
 	    (if noerror
 		nil
@@ -199,5 +201,4 @@ IDUtils must start from scratch when updating a database."
 
 (provide 'cedet-idutils)
 
-;; arch-tag: 663ca082-5b3d-4384-8710-cc74f990b501
 ;;; cedet-idutils.el ends here
