@@ -1,11 +1,11 @@
-;;; authors.el --- utility for maintaining Emacs' AUTHORS file -*-coding: utf-8;-*-
+;;; authors.el --- utility for maintaining Emacs's AUTHORS file -*-coding: utf-8 -*-
 
-;; Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-;;   2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 2000-2013 Free Software Foundation, Inc.
 
 ;; Author: Gerd Moellmann <gerd@gnu.org>
 ;; Maintainer: Kim F. Storm <storm@cua.dk>
 ;; Keywords: maint
+;; Package: emacs
 
 ;; This file is part of GNU Emacs.
 
@@ -25,7 +25,7 @@
 ;;; Commentary:
 
 ;; Use M-x authors RET to create an *Authors* buffer that can used as
-;; or merged with Emacs' AUTHORS file.
+;; or merged with Emacs's AUTHORS file.
 
 ;;; Code:
 
@@ -41,23 +41,28 @@ files.")
 (defconst authors-aliases
   '(
     ("Aaron S. Hawley" "Aaron Hawley")
+    ("Alexandru Harsanyi" "Alex Harsanyi")
     ("Andrew Csillag" "Drew Csillag")
     ("Anna M. Bigatti" "Anna Bigatti")
     ("Barry A. Warsaw" "Barry A. Warsaw, Century Computing, Inc."
      "Barry A. Warsaw, ITB" "Barry Warsaw")
+    ("Bill Carpenter" "WJ Carpenter")
     ("Bill Mann" "William F. Mann")
     ("Bill Rozas" "Guillermo J. Rozas")
     ("Björn Torkelsson" "Bjorn Torkelsson")
     ("Brian Fox" "Brian J. Fox")
+    ("Brian Sniffen" "Brian T. Sniffen")
     ("Christoph Wedler" "Christoph.Wedler@sap.com")
     ("Daniel Pfeiffer" "<Daniel.Pfeiffer@Informatik.START.db.de>"
      "<Daniel.Pfeiffer@Informatik.START.dbp.de>")
+    ("David Abrahams" "Dave Abrahams")
     ("David De La Harpe Golden" "David Golden")
     ("David Gillespie" "Dave Gillespie")
     ("David Kågedal" "David K..edal")
     ("David M. Koppelman" "David M. Koppelman, Koppel@Ec?e.Lsu.Edu"
      "David Koppelman")
     ("David M. Smith" "David Smith" "David M Smith")
+    ("David O'Toole" "David T. O'Toole")
     ("Deepak Goel" "D. Goel")
     ("Ed L. Cashin" "Ed L Cashin")
     ("Edward M. Reingold" "Ed Reingold" "Edward M Reingold"
@@ -76,6 +81,8 @@ files.")
     ("Gerd Möllmann" "Gerd Moellmann")
     ("Hallvard B. Furuseth" "Hallvard B Furuseth" "Hallvard Furuseth")
     ("Hrvoje Nikšić" "Hrvoje Niksic")
+    ;; lisp/org/ChangeLog 2010-11-11.
+    (nil "aaa bbb")
     ;; src/ChangeLog.4, 1994-01-11, since fixed.
 ;;;    (nil "(afs@hplb.hpl.hp.com)")
     ;; lisp/gnus/ChangeLog.1, 1998-01-15.
@@ -92,6 +99,7 @@ files.")
     ("Jens-Ulrik Holger Petersen" "Jens-Ulrik Petersen")
     ("Jeremy Bertram Maitin-Shepard" "Jeremy Maitin-Shepard")
     ("Johan Bockgård" "Johan Bockgard")
+    ("John J Foerch" "John Foerch")
     ("John W. Eaton" "John Eaton")
     ("Jonathan I. Kamens" "Jonathan Kamens")
     ("Joseph Arceneaux" "Joe Arceneaux")
@@ -109,10 +117,11 @@ files.")
     ("Kevin Greiner" "Kevin J. Greiner")
     ("Kim F. Storm" "Kim Storm")
     ("Kyle Jones" "Kyle E. Jones")
+    ("Lars Magne Ingebrigtsen" "Lars Ingebrigtsen")
     ("Marcus G. Daniels" "Marcus Daniels")
     ("Mark D. Baushke" "Mark D Baushke")
     ("Marko Kohtala" "Kohtala Marko")
-    ("Agustín Martín" "Agustin Martin")
+    ("Agustín Martín" "Agustin Martin" "Agustín Martín Domingo")
     ("Martin Lorentzon" "Martin Lorentzson")
     ("Matt Swift" "Matthew Swift")
     ("Maxime Edouard Robert Froumentin" "Max Froumentin")
@@ -124,12 +133,14 @@ files.")
     ("Michael Sperber" "Michael Sperber \\[Mr. Preprocessor\\]")
     ("Mikio Nakajima" "Nakajima Mikio")
     ("Nelson Jose dos Santos Ferreira" "Nelson Ferreira")
+    ("Noorul Islam" "Noorul Islam K M")
     ("Paul Eggert" "eggert")
     ("Paul Reilly" "(pmr@legacy.pajato.com)")
     ("Pavel Janík" "Pavel Janík Ml." "Pavel Janik Ml." "Pavel Janik" "Pavel Janík" "Pavel@Janik.Cz")
     ("Pavel Kobiakov" "Pavel Kobyakov")
     ("Per Abrahamsen" "Per Abhiddenware")
     ("Per Starbäck" "Per Starback")
+    ("Peter J. Weisberg" "PJ Weisberg")
     ("Peter S. Galbraith" "Peter Galbraith")
     ("Peter Runestig" "Peter 'luna' Runestig")
     ("Peter S. Galbraith" "Peter S Galbraith")
@@ -144,23 +155,28 @@ files.")
     ("Sacha Chua" "Sandra Jean Chua")
     ("Sam Steingold" "Sam Shteingold")
     ("Satyaki Das" "Indexed search by Satyaki Das")
+    ("Sébastien Vauban" "Sebastien Vauban")
     ;; There are other Stefans.
 ;;;    ("Stefan Monnier" "Stefan")
     ("Stephen A. Wood" "(saw@cebaf.gov)")
     ("Steven L. Baur" "SL Baur" "Steven L Baur")
     ("Stewart M. Clamen" "Stewart Clamen")
     ("Stuart D. Herring" "Stuart Herring" "Davis Herring")
+    ("T.V. Raman" "T\\. V\\. Raman")
     ("Taichi Kawabata" "KAWABATA,? Taichi")
     ("Takaaki Ota" "Tak Ota")
     ("Takahashi Naoto" "Naoto Takahashi")
     ("Teodor Zlatanov" "Ted Zlatanov")
+    ("Thomas Dye" "Tom Dye")
     ("Thomas Horsley" "Tom Horsley")	; FIXME ?
     ("Thomas Wurgler" "Tom Wurgler")
+    ("Toby Cubitt" "Toby S\\. Cubitt")
     ("Tomohiko Morioka" "MORIOKA Tomohiko")
     ("Torbjörn Axelsson" "Torbjvrn Axelsson")
     ("Torbjörn Einarsson" "Torbj.*rn Einarsson")
     ("Toru Tomabechi" "Toru Tomabechi,")
     ("Tsugutomo Enami" "enami tsugutomo")
+    ("Ulrich Müller" "Ulrich Mueller")
     ("Vincent Del Vecchio" "Vince Del Vecchio")
     ("William M. Perry" "Bill Perry")
     ("Wlodzimierz Bzyl" "W.*dek Bzyl")
@@ -174,6 +190,7 @@ Each entry is of the form (REALNAME REGEXP...).  If an author's name
 matches one of the REGEXPs, use REALNAME instead.
 If REALNAME is nil, ignore that author.")
 
+;; FIXME seems it would be less fragile to check for O', Mc, etc.
 (defconst authors-fixed-case
   '("Bryan O'Sullivan"
     "Christian von Roques"
@@ -182,6 +199,7 @@ If REALNAME is nil, ignore that author.")
     "David J. MacKenzie"
     "David McCabe"
     "David O'Toole"
+    "Devon Sean McCullough"
     "Dominique de Waleffe"
     "Edward O'Connor"
     "Exal de Jesus Garcia Carrillo"
@@ -194,8 +212,11 @@ If REALNAME is nil, ignore that author.")
     "Nelson Jose dos Santos Ferreira"
     "Peter von der Ahe"
     "Peter O'Gorman"
+    "Piet van Oostrum"
     "Roland McGrath"
-    "Sean O'Rourke")
+    "Sean O'Halpin"
+    "Sean O'Rourke"
+    "Tijs van Bakel")
   "List of authors whose names cannot be simply capitalized.")
 
 (defvar authors-public-domain-files
@@ -220,10 +241,14 @@ If REALNAME is nil, ignore that author.")
   '("vc-\\*\\.el$"
     "spec.txt$"
     ".*loaddefs.el$"			; not obsolete, but auto-generated
+    "\\.\\(cvs\\|git\\)ignore$"		; obsolete or uninteresting
+    "\\.arch-inventory$"
+    ;; TODO lib/? Matches other things?
+    "build-aux/" "m4/" "Emacs.xcodeproj" "charsets" "mapfiles"
+    "preferences\\.\\(nib\\|gorm\\)"
     "vc-\\(rcs\\|cvs\\|sccs\\)-hooks\\.el$")
   "List of regexps matching obsolete files.
-Changes to files matching one of the regexps in this list are not
-listed.")
+Changes to files matching one of the regexps in this list are not listed.")
 
 (defconst authors-ignored-files
   '("external-lisp"
@@ -244,9 +269,22 @@ listed.")
     "Imakefile" "icons/sink.ico" "aixcc.lex"
     "nxml/char-name/unicode"
     "js2-mode.el"      ; only installed very briefly, replaced by js.el
+    "cedet/tests/testtemplates.cpp"
+    "cedet/tests/testusing.cpp"
+    "cedet/tests/scopetest.cpp"
+    "cedet/tests/scopetest.java"
+    "cedet/tests/test.cpp"
+    "cedet/tests/test.py"
+    "cedet/tests/teststruct.cpp"
+    "*.el"
     ;; Autogen:
     "cus-load.el" "finder-inf.el" "ldefs-boot.el"
+    "compile" "config.guess" "config.sub" "depcomp"
+    ;; Only existed briefly, then renamed:
+    "images/icons/allout-widgets-dark-bg"
+    "images/icons/allout-widgets-light-bg"
     ;; Never had any meaningful changes logged, now deleted:
+    "unidata/bidimirror.awk" "unidata/biditype.awk"
     "split-man" "Xkeymap.txt" "ms-7bkermit" "ulimit.hack"
     "gnu-hp300" "refcard.bit" "ledit.l" "forms.README" "forms-d2.dat"
     "CXTERM-DIC/PY.tit" "CXTERM-DIC/ZIRANMA.tit"
@@ -255,7 +293,16 @@ listed.")
     "3B-MAXMEM" "AIX.DUMP" "SUN-SUPPORT" "XENIX"
     "CODINGS" "CHARSETS"
     "calc/INSTALL" "calc/Makefile"
-    "vms-pp.trans" "_emacs" "batcomp.com"
+    "vms-pp.trans" "_emacs" "batcomp.com" "notes/cpp" ; admin/
+    "emacsver.texi.in"
+    "vpath.sed"
+    "Cocoa/Emacs.base/Contents/Info.plist"
+    "Cocoa/Emacs.base/Contents/Resources/English.lproj/InfoPlist.strings"
+    "GNUstep/Emacs.base/Resources/Info-gnustep.plist"
+    "GNUstep/Emacs.base/Resources/Emacs.desktop"
+    "Cocoa/Emacs.base/Contents/Resources/English.lproj"
+    ;; Only existed briefly, then deleted:
+    "coccinelle/overlay.cocci" "coccinelle/symbol.cocci"
     ;; MH-E stuff not in Emacs:
     "import-emacs" "release-utils"
     ;; Erc stuff not in Emacs:
@@ -286,6 +333,60 @@ listed.")
   "List of files and directories to ignore.
 Changes to files in this list are not listed.")
 
+;; List via: find . -name '*.el' | sed 's/.*\///g' | sort | uniq -d
+;; FIXME It would be better to discover these dynamically.
+;; Note that traditionally "Makefile.in" etc have not been in this list.
+;; Ditto for "abbrev.texi" etc.
+(defconst authors-ambiguous-files
+  '("chart.el"
+    "compile.el"
+    "complete.el"
+    "cpp.el"
+    "ctxt.el"
+    "custom.el"
+    "cyrillic.el"
+    "czech.el"
+    "debug.el"
+    "dired.el"
+    "el.el"
+    "eshell.el"
+    "ethiopic.el"
+    "f90.el"
+    "files.el"
+    "find.el"
+    "format.el"
+    "generic.el"
+    "georgian.el"
+    "greek.el"
+    "grep.el"
+    "hebrew.el"
+    "imenu.el"
+    "indian.el"
+    "japanese.el"
+    "java.el"
+    "lao.el"
+    "linux.el"
+    "locate.el"
+    "make.el"
+    "mode.el"
+    "python.el"
+    "rmailmm.el"
+    "semantic.el"
+    "shell.el"
+    "simple.el"
+    "slovak.el"
+    "sort.el"
+    "speedbar.el"
+    "srecode.el"
+    "table.el"
+    "texi.el"
+    "thai.el"
+    "tibetan.el"
+    "util.el"
+    "vc-bzr.el"
+    "wisent.el")
+  "List of basenames occurring more than once in the source.")
+
 ;; FIXME :cowrote entries here can be overwritten by :wrote entries
 ;; derived from a file's Author: header (eg mh-e).  This really means
 ;; the Author: header is erroneous.
@@ -306,8 +407,8 @@ Changes to files in this list are not listed.")
     ("Lawrence R. Dodd" :cowrote "dired-x.el")
     ;; No longer distributed.
 ;;;    ("Viktor Dukhovni" :wrote "unexsunos4.c")
-    ("Paul Eggert" :wrote "rcs2log" "vcdiff")
-    ("Fred Fish" :changed "unexec.c")
+    ("Paul Eggert" :wrote "rcs2log") ; "vcdiff"
+    ("Fred Fish" :changed "unexcoff.c")
     ;; No longer distributed.
 ;;;    ("Tim Fleehart" :wrote "makefile.nt")
     ("Keith Gabryelski" :wrote "hexl.c")
@@ -330,13 +431,13 @@ Changes to files in this list are not listed.")
      "indent.c" "search.c" "xdisp.c" "region-cache.c" "region-cache.h")
     ;; ibmrt.h, ibmrt-aix.h no longer distributed.
     ("International Business Machines" :changed "emacs.c" "fileio.c"
-     "process.c" "sysdep.c" "unexec.c")
+     "process.c" "sysdep.c" "unexcoff.c")
     ;; No longer distributed.
 ;;;    ("Ishikawa Chiaki" :changed "aviion.h" "dgux.h")
     ;; ymakefile no longer distributed.
-    ("Michael K. Johnson" :changed "configure.in" "emacs.c" "intel386.h"
+    ("Michael K. Johnson" :changed "configure.ac" "emacs.c" "intel386.h"
      "mem-limits.h" "process.c" "template.h" "sysdep.c" "syssignal.h"
-     "systty.h" "unexec.c" "linux.h")
+     "systty.h" "unexcoff.c" "linux.h")
     ;; No longer distributed.
 ;;;    ("Kyle Jones" :wrote "mldrag.el")
     ("Henry Kautz" :wrote "bib-mode.el")
@@ -361,7 +462,7 @@ Changes to files in this list are not listed.")
      "rmail.el" "rmailedit.el" "rmailkwd.el"
      "rmailmsc.el" "rmailout.el" "rmailsum.el" "scribe.el"
      ;; It was :wrote for xmenu.c, but it has been rewritten since.
-     "server.el" "lisp.h" "sysdep.c" "unexec.c" "xmenu.c")
+     "server.el" "lisp.h" "sysdep.c" "unexcoff.c" "xmenu.c")
     ("Niall Mansfield" :changed "etags.c")
     ("Brian Marick" :cowrote "hideif.el")
     ("Marko Kohtala" :changed "info.el")
@@ -373,7 +474,8 @@ Changes to files in this list are not listed.")
      "vt220.el" "vt240.el")
     ("Motorola" :changed "buff-menu.el")
     ("Hiroshi Nakano" :changed "ralloc.c")
-    ("Sundar Narasimhan" :changed "rnewspost.el")
+    ;; File removed in Emacs 24.1.
+;;;    ("Sundar Narasimhan" :changed "rnewspost.el")
     ;; No longer distributed.
 ;;;    ("NeXT, Inc." :wrote "unexnext.c")
     ("Mark Neale" :changed "fortran.el")
@@ -389,7 +491,7 @@ Changes to files in this list are not listed.")
      ;; No longer distributed.
 ;;;     "vmspaths.h" "build.com" "compile.com" "kepteditor.com" "precomp.com"
 ;;;     "vmsproc.el" :wrote "logout.com" "mailemacs.com")
-    ("Guillermo J. Rozas" :wrote "fakemail.c")
+;;;    ("Guillermo J. Rozas" :wrote "fakemail.c")
     ("Wolfgang Rupprecht" :changed "lisp-mode.el" "loadup.el"
      "sort.el" "alloc.c" "callint.c"
      ;; config.in renamed from config.h.in; ecrt0.c from crt0.c.
@@ -404,7 +506,7 @@ Changes to files in this list are not listed.")
     ("William Sommerfeld" :wrote "emacsclient.c" "scribe.el")
     ;; No longer distributed: emacsserver.c.
     ("Leigh Stoller" :changed "emacsclient.c" "server.el")
-    ("Steve Strassman" :wrote "spook.el")
+    ("Steve Strassmann" :wrote "spook.el")
     ("Shinichirou Sugou" :changed "etags.c")
     ;; No longer distributed: emacsserver.c.
     ("Sun Microsystems, Inc" :changed "emacsclient.c" "server.el"
@@ -416,9 +518,9 @@ Changes to files in this list are not listed.")
     ("Kayvan Sylvan" :changed "supercite.el")
     ;; No longer distributed: emacsserver.c, tcp.c.
     ("Spencer Thomas" :changed "emacsclient.c" "server.el"
-     "dabbrev.el" "unexec.c" "gnus.texi")
+     "dabbrev.el" "unexcoff.c" "gnus.texi")
     ("Jonathan Vail" :changed "vc.el")
-    ("James Van Artsdalen" :changed "usg5-4.h" "unexec.c")
+    ("James Van Artsdalen" :changed "usg5-4.h" "unexcoff.c")
     ;; No longer distributed: src/makefile.nt, lisp/makefile.nt
     ;; winnt.el renamed to w32-fns.el; nt.[ch] to w32.[ch];
     ;; ntheap.[ch] to w32heap.[ch]; ntinevt.c to w32inevt.c;
@@ -427,6 +529,7 @@ Changes to files in this list are not listed.")
     ("Geoff Voelker" :wrote "w32-fns.el" "w32.c" "w32.h" "w32heap.c"
      "w32heap.h" "w32inevt.c" "w32proc.c" "w32term.c" "ms-w32.h")
     ("Morten Welinder" :wrote "dosfns.c" "[many MS-DOS files]" "msdos.h")
+    ("Eli Zaretskii" :wrote "bidi.c" "[bidirectional display in xdisp.c]")
     ;; Not using this version any more.
 ;;;    ("Pace Willisson" :wrote "ispell.el")
     ;; FIXME overwritten by Author:.
@@ -445,6 +548,7 @@ Changes to files in this list are not listed.")
     "makedist.bat"
     "makefile.def"
     "makefile.nt"
+    "ns.mk"
     "debug.bat.in" "emacs.bat.in"
     ".gdbinit-union"
     "alloca.s"
@@ -457,17 +561,25 @@ Changes to files in this list are not listed.")
     "getdate.y"
     "ymakefile"
     "permute-index" "index.perm"
+    "ibmrs6000.inp"
+    "b2m.c" "b2m.1" "b2m.pl" "rcs-checkin.1"
+    "emacs.bash" "emacs.csh" "ms-kermit"
     "emacs.ico"
     "emacs21.ico"
-    "LPF" "LEDIT" "OTHER.EMACSES"
+    "emacs.py" "emacs2.py" "emacs3.py"
+    "BABYL" "LPF" "LEDIT" "OTHER.EMACSES"
     "emacs16_mac.png" "emacs24_mac.png"
     "emacs256_mac.png" "emacs32_mac.png"
     "emacs48_mac.png" "emacs512_mac.png"
+    "revdiff"				; admin/
+    "vcdiff" "rcs-checkin" "tindex.pl"
+    "mainmake" "sed1.inp" "sed2.inp" "sed3.inp" ; msdos/
+    "mac-fix-env.m"
     ;; Deleted vms stuff:
     "temacs.opt" "descrip.mms" "compile.com" "link.com"
     )
-  "File names which are valid, but no longer exist (or cannot be
-found) in the repository.")
+  "File names which are valid, but no longer exist (or cannot be found)
+in the repository.")
 
 (defconst authors-renamed-files-alist
   '(("nt.c" . "w32.c") ("nt.h" . "w32.h")
@@ -477,8 +589,11 @@ found) in the repository.")
     ("w32console.c" . "w32term.c")
     ("unexnt.c" . "unexw32.c")
     ("s/windowsnt.h" . "s/ms-w32.h")
+    ("s/ms-w32.h" . "inc/ms-w32.h")
     ("winnt.el" . "w32-fns.el")
+    ("emacs.manifest" . "emacs-x86.manifest")
     ("config.emacs" . "configure")
+    ("configure.in" . "configure.ac")
     ("config.h.dist" . "config.in")
     ("config.h-dist" . "config.in")
     ("config.h.in" . "config.in")
@@ -504,6 +619,7 @@ found) in the repository.")
     ;; index and pick merged into search.
     ("mh-index.el" . "mh-search.el")
     ("mh-pick.el" . "mh-search.el")
+    ("font-setting.el" . "dynamic-setting.el")
     ;; INSTALL-CVS -> .CVS -> .BZR
     ("INSTALL-CVS" . "INSTALL.BZR")
     ("INSTALL.CVS" . "INSTALL.BZR")
@@ -512,6 +628,8 @@ found) in the repository.")
     ("build-install" . "build-ins.in")
     ("build-install.in" . "build-ins.in")
     ("unidata/Makefile" . "unidata/Makefile.in")
+    ("move-if-change" . "build-aux/move-if-change")
+    ("update-subdirs" . "build-aux/update-subdirs")
     ;; Not renamed, but we only have the latter in the Emacs repo.
     ("trampver.texi.in" . "trampver.texi")
     ("e/eterm" . "e/eterm-color")
@@ -529,12 +647,17 @@ found) in the repository.")
     ("schema/docbook-dyntbl.rnc" . "schema/docbk-dyntbl.rnc")
     ("schema/docbook-soextbl.rnc" . "schema/docbk-soextbl.rn" )
     ("texi/url.txi" . "url.texi")
+    ("edt-user.doc" . "edt.texi")
+    ("DEV-NOTES" . "nextstep")
+    ("org/COPYRIGHT-AND-LICENSE" . "org/README")
     ;; Moved to different directories.
     ("ctags.1" . "ctags.1")
     ("etags.1" . "etags.1")
     ("emacs.1" . "emacs.1")
     ("emacsclient.1" . "emacsclient.1")
     ("icons/emacs21.ico" . "emacs21.ico")
+    ;; Moved from admin/nt/ to nt/.
+    ("nt/README.W32" . "README.W32")
     )
   "Alist of files which have been renamed during their lifetime.
 Elements are (OLDNAME . NEWNAME).")
@@ -573,14 +696,29 @@ Otherwise, the file name is accepted as is.")
 (defvar authors-checked-files-alist)
 (defvar authors-invalid-file-names)
 
+(defun authors-disambiguate-file-name (fullname)
+  "Convert FULLNAME to an unambiguous relative-name."
+  (let ((relname (file-name-nondirectory fullname))
+	parent)
+    (if (member relname authors-ambiguous-files)
+	;; In case of ambiguity, just prepend the parent directory.
+	;; FIXME obviously this is not a perfect solution.
+	(if (string-equal "lisp"
+			  (setq parent (file-name-nondirectory
+					(directory-file-name
+					 (file-name-directory fullname)))))
+	    relname
+	  (format "%s/%s" parent relname))
+      relname)))
+
 (defun authors-canonical-file-name (file log-file pos author)
   "Return canonical file name for FILE found in LOG-FILE.
 Checks whether FILE is a valid (existing) file name, has been renamed,
-or is on the list of removed files.  Returns the non-diretory part of
+or is on the list of removed files.  Returns the non-directory part of
 the file name.  Only uses the LOG-FILE position POS and associated AUTHOR
 to print a message if FILE is not found."
   ;; FILE should be re-checked in every different directory associated
-  ;; with a LOG-FILE.  Eg configure.in from src/ChangeLog is not the
+  ;; with a LOG-FILE.  Eg configure.ac from src/ChangeLog is not the
   ;; same as that from top-level/ChangeLog.
   (let* ((fullname (expand-file-name file (file-name-directory log-file)))
 	 (entry (assoc fullname authors-checked-files-alist))
@@ -593,7 +731,7 @@ to print a message if FILE is not found."
 	      (file-exists-p file)
 	      (file-exists-p relname)
 	      (file-exists-p (concat "etc/" relname)))
-	  (setq valid relname)
+	  (setq valid (authors-disambiguate-file-name fullname))
 	(setq valid (assoc file authors-renamed-files-alist))
 	(if valid
 	    (setq valid (cdr valid))
@@ -610,6 +748,7 @@ to print a message if FILE is not found."
 	    (cons (cons fullname valid) authors-checked-files-alist))
       (unless (or valid
 		  (member file authors-ignored-files)
+		  (authors-obsolete-file-p file)
 		  (string-match "[*]" file)
 		  (string-match "^[0-9.]+$" file))
 	(setq authors-invalid-file-names
@@ -707,7 +846,7 @@ with the file and the number of each action:
 	 (enable-local-eval nil)
 	 (existing-buffer (get-file-buffer log-file))
 	 (buffer (find-file-noselect log-file))
-	 authors file pos)
+	 authors pos)
     (with-current-buffer buffer
       (save-restriction
 	(widen)
@@ -758,7 +897,7 @@ TABLE is a hash table to add author information to."
 	 (enable-local-variables :safe)	; for find-file, hence let*
 	 (enable-local-eval nil)
 	 (buffer (find-file-noselect file)))
-    (setq file (file-name-nondirectory file))
+    (setq file (authors-disambiguate-file-name (expand-file-name file)))
     (with-current-buffer buffer
       (save-restriction
 	(widen)
@@ -821,8 +960,7 @@ and changed by AUTHOR."
 	       (file (car change))
 	       (filestat (if (authors-public-domain-p file)
 			     (concat file " (public domain)")
-			   file))
-	       slot)
+			   file)))
 	  (cond ((assq :wrote actions)
 		 (setq wrote-list (cons filestat wrote-list)))
 		((assq :cowrote actions)
@@ -956,5 +1094,4 @@ the Emacs source tree, from which to build the file."
 
 (provide 'authors)
 
-;; arch-tag: 659d5900-5ff2-43b0-954c-a315cc1e4dc1
 ;;; authors.el ends here
