@@ -1,7 +1,6 @@
 ;;; hippie-exp.el --- expand text trying various ways to find its expansion
 
-;; Copyright (C) 1992, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-;;   2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 1992, 2001-2013 Free Software Foundation, Inc.
 
 ;; Author: Anders Holst <aho@sans.kth.se>
 ;; Last change: 3 March 1998
@@ -157,7 +156,7 @@
 ;;  opposite situation to occur, that `hippie-expand' misses some
 ;;  suggestion because it thinks it has already tried it.
 ;;
-;;  Acknowledgement
+;;  Acknowledgment
 ;;
 ;;  I want to thank Mikael Djurfeldt in discussions with whom the idea
 ;;  of this function took form.
@@ -200,6 +199,7 @@
 
 (defvar he-search-window ())
 
+;;; Autoloaded for historical reasons (bug#12982)
 ;;;###autoload
 (defcustom hippie-expand-try-functions-list
   '(try-complete-file-name-partially
@@ -218,31 +218,26 @@ or insert functions in this list."
   :type '(repeat function)
   :group 'hippie-expand)
 
-;;;###autoload
 (defcustom hippie-expand-verbose t
   "Non-nil makes `hippie-expand' output which function it is trying."
   :type 'boolean
   :group 'hippie-expand)
 
-;;;###autoload
 (defcustom hippie-expand-dabbrev-skip-space nil
   "Non-nil means tolerate trailing spaces in the abbreviation to expand."
   :group 'hippie-expand
   :type 'boolean)
 
-;;;###autoload
 (defcustom hippie-expand-dabbrev-as-symbol t
   "Non-nil means expand as symbols, i.e. syntax `_' is considered a letter."
   :group 'hippie-expand
   :type 'boolean)
 
-;;;###autoload
 (defcustom hippie-expand-no-restriction t
   "Non-nil means that narrowed buffers are widened during search."
   :group 'hippie-expand
   :type 'boolean)
 
-;;;###autoload
 (defcustom hippie-expand-max-buffers ()
   "The maximum number of buffers (apart from the current) searched.
 If nil, all buffers are searched."
@@ -250,15 +245,13 @@ If nil, all buffers are searched."
 		 integer)
   :group 'hippie-expand)
 
-;;;###autoload
-(defcustom hippie-expand-ignore-buffers (list (purecopy "^ \\*.*\\*$") 'dired-mode)
+(defcustom hippie-expand-ignore-buffers '("^ \\*.*\\*$" dired-mode)
   "A list specifying which buffers not to search (if not current).
 Can contain both regexps matching buffer names (as strings) and major modes
 \(as atoms)"
   :type '(repeat (choice regexp (symbol :tag "Major Mode")))
   :group 'hippie-expand)
 
-;;;###autoload
 (defcustom hippie-expand-only-buffers ()
   "A list specifying the only buffers to search (in addition to current).
 Can contain both regexps matching buffer names (as strings) and major modes
@@ -392,7 +385,7 @@ undoes the expansion."
 	      lst)))
 
 ;; Check if current buffer matches any atom or regexp in LST.
-;; Atoms are interpreted as major modes, strings as regexps mathing the name.
+;; Atoms are interpreted as major modes, strings as regexps matching the name.
 (defun he-buffer-member (lst)
   (or (memq major-mode lst)
       (progn
@@ -716,8 +709,7 @@ string).  It returns t if a new completion is found, nil otherwise."
 (defun he-line-beg (strip-prompt)
   (save-excursion
     (if (re-search-backward (he-line-search-regexp "" strip-prompt)
-			    (save-excursion (beginning-of-line)
-					    (point)) t)
+			    (line-beginning-position) t)
 	(match-beginning 2)
       (point))))
 
@@ -1184,5 +1176,4 @@ string).  It returns t if a new completion is found, nil otherwise."
 
 (provide 'hippie-exp)
 
-;; arch-tag: 5e6e00bf-b061-4a7a-9b46-de0ae105ab99
 ;;; hippie-exp.el ends here
