@@ -1,7 +1,6 @@
 ;;; mule-util.el --- utility functions for multilingual environment (mule)
 
-;; Copyright (C) 1997, 1998, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 1997-1998, 2000-2013 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -34,39 +33,6 @@
 ;;; String manipulations while paying attention to multibyte
 ;;; characters.
 
-;;;###autoload
-(defun string-to-sequence (string type)
-  "Convert STRING to a sequence of TYPE which contains characters in STRING.
-TYPE should be `list' or `vector'."
-;;;  (let ((len (length string))
-;;;	   (i 0)
-;;;	   val)
-    (cond ((eq type 'list)
-	   ;; Applicable post-Emacs 20.2 and asymptotically ~10 times
-	   ;; faster than the code below:
-	   (append string nil))
-;;; 	   (setq val (make-list len 0))
-;;; 	   (let ((l val))
-;;; 	     (while (< i len)
-;;; 	       (setcar l (aref string i))
-;;; 	       (setq l (cdr l) i (1+ i))))))
-	  ((eq type 'vector)
-	   ;; As above.
-	   (vconcat string))
-;;; 	   (setq val (make-vector len 0))
-;;; 	   (while (< i len)
-;;; 	     (aset val i (aref string i))
-;;; 	     (setq i (1+ i))))
-	  (t
-	   (error "Invalid type: %s" type)))
-;;;    val)
-)
-
-;;;###autoload
-(make-obsolete 'string-to-sequence
-	       "use `string-to-list' or `string-to-vector'."
-	       "22.1")
-
 ;;;###autoload
 (defsubst string-to-list (string)
   "Return a list of characters in STRING."
@@ -314,8 +280,8 @@ Optional 5th argument NIL-FOR-TOO-LONG non-nil means return nil
 ;;;###autoload
 (defmacro with-coding-priority (coding-systems &rest body)
   "Execute BODY like `progn' with CODING-SYSTEMS at the front of priority list.
-CODING-SYSTEMS is a list of coding systems.  See `set-coding-priority'.
-This affects the implicit sorting of lists of coding sysems returned by
+CODING-SYSTEMS is a list of coding systems.  See `set-coding-system-priority'.
+This affects the implicit sorting of lists of coding systems returned by
 operations such as `find-coding-systems-region'."
   (let ((current (make-symbol "current")))
   `(let ((,current (coding-system-priority-list)))
@@ -331,10 +297,9 @@ operations such as `find-coding-systems-region'."
   "Detect a coding system of the text between FROM and TO with PRIORITY-LIST.
 PRIORITY-LIST is an alist of coding categories vs the corresponding
 coding systems ordered by priority."
+  (declare (obsolete with-coding-priority "23.1"))
   `(with-coding-priority (mapcar #'cdr ,priority-list)
      (detect-coding-region ,from ,to)))
-(make-obsolete 'detect-coding-with-priority
-	       "use `with-coding-priority' and `detect-coding-region'." "23.1")
 
 ;;;###autoload
 (defun detect-coding-with-language-environment (from to lang-env)
@@ -405,5 +370,4 @@ per-character basis, this may not be accurate."
 ;; coding: iso-2022-7bit
 ;; End:
 
-;; arch-tag: 5bdb52b6-a3a5-4529-b7a0-37d01b0e570b
 ;;; mule-util.el ends here
