@@ -1,18 +1,18 @@
 ;;; ogonek.el --- change the encoding of Polish diacritics
 
-;; Copyright (C) 1997, 1998, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 1997-1998, 2001-2013 Free Software Foundation, Inc.
 
-;; Author: W{\l}odek Bzyl, Ryszard Kubiak
-;; Maintainer: rysiek@ipipan.gda.pl (Ryszard Kubiak)
+;; Author: W{\l}odek Bzyl
+;;	   Ryszard Kubiak
+;; Maintainer: Ryszard Kubiak <rysiek@ipipan.gda.pl>
 ;; Keywords: i18n
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -226,13 +224,14 @@ The functions come in the following groups.
    ogonek-prefix-to-encoding      iso8859-2
 
  The above default values can be changed by placing appropriate settings
- in the '~/.emacs' file:
+ in your init file:
 
    (setq ogonek-prefix-char ?/)
    (setq ogonek-prefix-to-encoding \"iso8859-2\")
 
- Instead of loading the whole library `ogonek' it may be better to
- autoload the needed functions, for example by placing in `~/.emacs':
+ Instead of loading the whole library `ogonek' it may be better
+ to autoload the needed functions, for example by adding the
+ following lines to your init file:
 
    (autoload 'ogonek-how \"ogonek\")
    (autoload 'ogonek-recode-region \"ogonek\")
@@ -274,23 +273,23 @@ The functions come in the following groups.
 		ogonek-name-encoding-alist))
   "List of ogonek encodings.  Used only for customization.")
 (defcustom ogonek-from-encoding "iso8859-2"
-  "*Encoding in the source file of recoding."
+  "Encoding in the source file of recoding."
   :type ogonek-encoding-choices
   :group 'ogonek)
 (defcustom ogonek-to-encoding "ascii"
-  "*Encoding in the target file of recoding."
+  "Encoding in the target file of recoding."
   :type ogonek-encoding-choices
   :group 'ogonek)
 (defcustom ogonek-prefix-char ?/
-  "*Prefix character for prefix encodings."
+  "Prefix character for prefix encodings."
   :type 'character
   :group 'ogonek)
 (defcustom ogonek-prefix-from-encoding "iso8859-2"
-  "*Encoding in the source file subject to prefixifation."
+  "Encoding in the source file subject to prefixation."
   :type ogonek-encoding-choices
   :group 'ogonek)
 (defcustom ogonek-prefix-to-encoding "iso8859-2"
-  "*Encoding in the target file subject to deprefixifation."
+  "Encoding in the target file subject to deprefixation."
   :type ogonek-encoding-choices
   :group 'ogonek)
 
@@ -501,5 +500,4 @@ followed by a non-Polish character, that is one not listed in the
 
 (provide 'ogonek)
 
-;;; arch-tag: 672d7744-28ac-412b-965e-06a27e50d1d7
 ;;; ogonek.el ends here
