@@ -1,7 +1,6 @@
 ;;; cyrillic.el --- support for Cyrillic -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 1997, 1998, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 1997-1998, 2001-2013 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -127,7 +126,7 @@ Support for Russian using koi8-r and the russian-computer input method.")
 (define-coding-system 'koi8-u
   "KOI8-U 8-bit encoding for Cyrillic (MIME: KOI8-U)"
   :coding-type 'charset
-  :mnemonic ?U 
+  :mnemonic ?U
   :charset-list '(koi8-u)
   :mime-charset 'koi8-u)
 
@@ -247,7 +246,7 @@ Support for Russian using koi8-r and the russian-computer input method.")
 	       (ctext-non-standard-encodings "microsoft-cp1251")
 	       (input-method . "bulgarian-bds")
 	       (documentation
-		. "Support for Bulgrian with windows-1251 character set."))
+		. "Support for Bulgarian with windows-1251 character set."))
  '("Cyrillic"))
 
 (set-language-info-alist
@@ -272,5 +271,4 @@ Support for Russian using koi8-r and the russian-computer input method.")
 
 (provide 'cyrillic)
 
-;; arch-tag: bda71ae0-ba41-4cb6-a6e0-1dff542313d3
 ;;; cyrillic.el ends here
