@@ -1,8 +1,7 @@
 ;;; mh-comp.el --- MH-E functions for composing and sending messages
 
-;; Copyright (C) 1993, 1995, 1997,
-;;   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1995, 1997, 2000-2013 Free Software Foundation,
+;; Inc.
 
 ;; Author: Bill Wohler <wohler@newt.com>
 ;; Maintainer: Bill Wohler <wohler@newt.com>
@@ -199,7 +198,8 @@ applications should use `mh-user-agent-compose'."
 ;;;###autoload
 (defun mh-user-agent-compose (&optional to subject other-headers continue
                                         switch-function yank-action
-                                        send-actions)
+                                        send-actions return-action
+                                        &rest ignored)
   "Set up mail composition draft with the MH mail system.
 This is the `mail-user-agent' entry point to MH-E. This function
 conforms to the contract specified by `define-mail-user-agent'
@@ -213,8 +213,8 @@ OTHER-HEADERS is an alist specifying additional header fields.
 Elements look like (HEADER . VALUE) where both HEADER and VALUE
 are strings.
 
-CONTINUE, SWITCH-FUNCTION, YANK-ACTION and SEND-ACTIONS are
-ignored."
+CONTINUE, SWITCH-FUNCTION, YANK-ACTION, SEND-ACTIONS, and
+RETURN-ACTION and any additional arguments are IGNORED."
   (mh-find-path)
   (let ((mh-error-if-no-draft t))
     (mh-send to "" subject)
@@ -224,7 +224,8 @@ ignored."
       (setq other-headers (cdr other-headers)))))
 
 ;; Shush compiler.
-(defvar sendmail-coding-system)         ; XEmacs
+(mh-do-in-xemacs
+  (defvar sendmail-coding-system))
 
 ;;;###autoload
 (defun mh-send-letter (&optional arg)
@@ -276,8 +277,8 @@ message and scan line."
                              '(undecided undecided-unix undecided-dos))))
              buffer-file-coding-system
            (or (and (boundp 'sendmail-coding-system) sendmail-coding-system)
-               (and (boundp 'default-buffer-file-coding-system )
-                    default-buffer-file-coding-system)
+               (and (default-boundp 'buffer-file-coding-system)
+                    (default-value 'buffer-file-coding-system))
                'iso-latin-1))))
     ;; Older versions of spost do not support -msgid and -mime.
     (unless mh-send-uses-spost-flag
@@ -473,8 +474,7 @@ See also `mh-compose-forward-as-mime-flag',
                        (mh-read-draft "" draft-name nil)))))
     (let (orig-from
           orig-subject)
-      (save-excursion
-        (set-buffer (get-buffer-create mh-temp-buffer))
+      (with-current-buffer (get-buffer-create mh-temp-buffer)
         (erase-buffer)
         (insert-file-contents fwd-msg-file)
         (setq orig-from (mh-get-header-field "From:"))
@@ -505,7 +505,7 @@ See also `mh-compose-forward-as-mime-flag',
                                             folder msg)
                     ;; Was inserted before us, move to end of file to preserve order
                     (goto-char (point-max)))))))
-        ;; Postition just before forwarded message
+        ;; Position just before forwarded message.
         (if (re-search-forward "^------- Forwarded Message" nil t)
             (forward-line -1)
           (goto-char (mh-mail-header-end))
@@ -906,6 +906,9 @@ letter."
   (mh-identity-make-menu)
   (mh-identity-add-menu)
 
+  ;; Cleanup possibly RFC2047 encoded subject header
+  (mh-decode-message-subject)
+
   ;; Insert extra fields.
   (mh-insert-x-mailer)
   (mh-insert-x-face)
@@ -1110,5 +1113,4 @@ doesn't exist there."
 ;; sentence-end-double-space: nil
 ;; End:
 
-;; arch-tag: 62865511-e610-4923-b0b5-f45a8ab70a34
 ;;; mh-comp.el ends here
