@@ -1,6 +1,6 @@
 ;;; zeroconf.el --- Service browser using Avahi.
 
-;; Copyright (C) 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 2008-2013 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, hardware
@@ -82,7 +82,7 @@
 
 ;; The function `zeroconf-publish-service' publishes a new service to
 ;; the Avahi daemon.  Although the domain, where to the service is
-;; published, can be specified by this function, it is usally the
+;; published, can be specified by this function, it is usually the
 ;; default domain "local" (also written as nil or "").
 
 ;;   (zeroconf-publish-service
@@ -102,11 +102,6 @@
 ;;  Pacify byte-compiler.  D-Bus support in the Emacs core can be
 ;; disabled with configuration option "--without-dbus".  Declare used
 ;; subroutines and variables of `dbus' therefore.
-(eval-when-compile
-  (require 'cl))
-
-(declare-function dbus-call-method "dbusbind.c")
-(declare-function dbus-register-signal "dbusbind.c")
 (defvar dbus-debug)
 
 (require 'dbus)
@@ -249,7 +244,7 @@ qualified name, i.e., it contains DOMAIN.
 APROTOCOL stands for the network protocol family ADDRESS is
 encoded (`zeroconf-avahi-protocol-inet4' means INET4,
 `zeroconf-avahi-protocol-inet6' means INET6).  It can be
-different from PROTOCOL, when an adrress resolution has been
+different from PROTOCOL, when an address resolution has been
 requested for another protocol family but the default one.
 
 ADDRESS is the service address, encoded according to the
@@ -486,7 +481,7 @@ TYPE. The resulting list has the format
   (nth 9 (zeroconf-resolve-service service)))
 
 
-;;; Services signalling.
+;;; Services signaling.
 
 ;; Register for the service type browser.  Service registrations will
 ;; happen in `zeroconf-service-type-browser-handler', when there is an
@@ -548,7 +543,7 @@ DOMAIN is nil, the local domain is used."
    ((string-equal (dbus-event-member-name last-input-event) "ItemNew")
     ;; Parameters: (interface protocol type domain flags)
     ;; Register a service browser.
-    (let ((object-path (zeroconf-register-service-browser (nth-value 2 val))))
+    (let ((object-path (zeroconf-register-service-browser (nth 2 val))))
       ;; Register the signals.
       (dolist (member '("ItemNew" "ItemRemove" "Failure"))
 	(dbus-register-signal
@@ -686,5 +681,4 @@ For the description of arguments, see `zeroconf-resolved-services-hash'."
 
 (provide 'zeroconf)
 
-;; arch-tag: ea578165-7fa8-44f4-90f0-de3940aec69f
 ;;; zeroconf.el ends here
