@@ -1,11 +1,9 @@
 ;;; reftex-global.el --- operations on entire documents with RefTeX
 
-;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 1997-2013 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <dominik@science.uva.nl>
 ;; Maintainer: auctex-devel@gnu.org
-;; Version: 4.31
 
 ;; This file is part of GNU Emacs.
 
@@ -139,7 +137,7 @@ No active TAGS table is required."
     (set (make-local-variable 'TeX-master) master)
     (erase-buffer)
     (insert "                MULTIPLE LABELS IN CURRENT DOCUMENT:\n")
-    (insert 
+    (insert
      " Move point to label and type `r' to run a query-replace on the label\n"
      " and its references.  Type `q' to exit this buffer.\n\n")
     (insert " LABEL               FILE\n")
@@ -190,15 +188,15 @@ This command should be used with care, in particular in multifile
 documents.  You should not use it if another document refers to this
 one with the `xr' package."
   (interactive)
-  ;; Resan the entire document
+  ;; Rescan the entire document
   (reftex-access-scan-info 1)
   ;; Get some insurance
   (if (and (reftex-is-multi)
            (not (yes-or-no-p "Replacing all simple labels in multiple files is risky.  Continue? ")))
       (error "Abort"))
   ;; Make the translation list
-  (let* ((re-core (concat "\\(" 
-                          (mapconcat 'cdr reftex-typekey-to-prefix-alist "\\|") 
+  (let* ((re-core (concat "\\("
+                          (mapconcat 'cdr reftex-typekey-to-prefix-alist "\\|")
                           "\\)"))
          (label-re (concat "\\`" re-core "\\([0-9]+\\)\\'"))
          (search-re (concat "[{,]\\(" re-core "\\([0-9]+\\)\\)[,}]"))
@@ -230,12 +228,12 @@ one with the `xr' package."
     ;; Save all document buffers before this operation
     (reftex-save-all-document-buffers)
 
-    ;; First test to check for erros
-    (setq n (reftex-translate 
+    ;; First test to check for errors.
+    (setq n (reftex-translate
              files search-re translate-alist error-fmt 'test))
 
     ;; Now the real thing.
-    (if (yes-or-no-p 
+    (if (yes-or-no-p
          (format "Replace %d items at %d places in %d files? "
                  (length translate-alist) n (length files)))
         (progn
@@ -253,9 +251,9 @@ one with the `xr' package."
 
 (defun reftex-translate (files search-re translate-alist error-fmt test)
   ;; In FILES, look for SEARCH-RE and replace match 1 of it with
-  ;; its association in TRANSLATE-ALSIT.  
+  ;; its association in TRANSLATE-ALIST.
   ;; If we do not find an association and TEST is non-nil, query
-  ;; to ignore the problematic string.  
+  ;; to ignore the problematic string.
   ;; If TEST is nil, it is ignored without query.
   ;; Return the number of replacements.
   (let ((n 0) file label match-data buf macro pos cell)
@@ -281,7 +279,7 @@ one with the `xr' package."
                          (or (looking-at "\\\\ref")
                              (looking-at "\\\\[a-zA-Z]*ref\\(range\\)?[^a-zA-Z]")
                              (looking-at "\\\\ref[a-zA-Z]*[^a-zA-Z]")
-                             (looking-at (format 
+                             (looking-at (format
                                           reftex-find-label-regexp-format
                                           (regexp-quote label)))))
                 ;; OK, we should replace it.
@@ -350,9 +348,8 @@ Also checks if buffers visiting the files are in read-only mode."
 ;; variable `multi-isearch-next-buffer-function'.
 
 (defun reftex-isearch-wrap-function ()
-  (if (not isearch-word)
-      (switch-to-buffer 
-       (funcall isearch-next-buffer-function (current-buffer) t)))
+  (switch-to-buffer
+   (funcall isearch-next-buffer-function (current-buffer) t))
   (goto-char (if isearch-forward (point-min) (point-max))))
 
 (defun reftex-isearch-push-state-function ()
@@ -364,14 +361,7 @@ Also checks if buffers visiting the files are in read-only mode."
 
 (defun reftex-isearch-isearch-search (string bound noerror)
   (let ((nxt-buff nil)
-	(search-fun
-	 (cond
-	  (isearch-word
-	   (if isearch-forward 'word-search-forward 'word-search-backward))
-	  (isearch-regexp
-	   (if isearch-forward 're-search-forward 're-search-backward))
-	  (t
-	   (if isearch-forward 'search-forward 'search-backward)))))
+	(search-fun (isearch-search-fun-default)))
     (or
      (funcall search-fun string bound noerror)
      (unless bound
@@ -415,7 +405,7 @@ Also checks if buffers visiting the files are in read-only mode."
     (when flist
       (if wrapp
 	  (unless isearch-forward
-	      (setq flist (last flist)))
+	    (setq flist (last flist)))
 	(unless isearch-forward
 	  (setq flist (reverse flist)))
 	(while (not (string= (car flist) cb))
@@ -435,7 +425,7 @@ With no argument, this command toggles
 `reftex-isearch-minor-mode' on if ARG is positive, otherwise turn it off."
   (interactive "P")
   (let ((old-reftex-isearch-minor-mode reftex-isearch-minor-mode))
-    (setq reftex-isearch-minor-mode 
+    (setq reftex-isearch-minor-mode
 	  (not (or (and (null arg) reftex-isearch-minor-mode)
 		   (<= (prefix-numeric-value arg) 0))))
     (unless (eq reftex-isearch-minor-mode old-reftex-isearch-minor-mode)
@@ -445,7 +435,8 @@ With no argument, this command toggles
 	      (with-current-buffer crt-buf
 		(when reftex-mode
 		  (if (boundp 'multi-isearch-next-buffer-function)
-		      (set (make-local-variable 'multi-isearch-next-buffer-function)
+		      (set (make-local-variable
+			    'multi-isearch-next-buffer-function)
 			   'reftex-isearch-switch-to-next-file)
 		    (set (make-local-variable 'isearch-wrap-function)
 			 'reftex-isearch-wrap-function)
@@ -468,11 +459,10 @@ With no argument, this command toggles
 		(kill-local-variable 'isearch-next-buffer-function))
 	      (setq reftex-isearch-minor-mode nil))))
 	(remove-hook 'reftex-mode-hook 'reftex-isearch-minor-mode)))
-    ;; Force modeline redisplay.
+    ;; Force mode line redisplay.
     (set-buffer-modified-p (buffer-modified-p))))
 
-(add-minor-mode 'reftex-isearch-minor-mode "/I" nil nil 
+(add-minor-mode 'reftex-isearch-minor-mode "/I" nil nil
 		'reftex-isearch-minor-mode)
 
-;; arch-tag: 2dbf7633-92c8-4340-8656-7aa019d0f80d
 ;;; reftex-global.el ends here
