@@ -1,5 +1,6 @@
 /* Flags and parameters describing user options for handling the terminal.
-   Copyright (C) 1985-1986, 1990, 2001-2012  Free Software Foundation, Inc.
+   Copyright (C) 1985-1986, 1990, 2001-2013 Free Software Foundation,
+   Inc.
 
 This file is part of GNU Emacs.
 
@@ -17,15 +18,8 @@ You should have received a copy of the GNU General Public License
 along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-/* Nonzero means use ^S/^Q as cretinous flow control.  */
-extern int flow_control;
-
 /* Nonzero means use interrupt-driven input.  */
-extern int interrupt_input;
+extern bool interrupt_input;
 
 /* Nonzero while interrupts are temporarily deferred during redisplay.  */
-extern int interrupts_deferred;
-
-/* Terminal has meta key */
-extern int meta_key;
-
+extern bool interrupts_deferred;
